# Extreme-oriented sensitivity analysis for train-track-bridge systems

Authors: Y. Shang and M. Nogal

Section of Integral Design and Management, Department of Materials, Mechanics, Management & Design, Faculty of Civil Engineering and Geosciences, Delft University of Technology

Contact Information:
y.shang@tudelft.nl, m.nogal@tudelft.nl

## General Information

The data and code files presented here are developed as part of Yue Shang's PhD Thesis project and are associated with the following publication. They are being made publicly available as supplementary data for the thesis of Yue Shang for review and reproduction purposes.

* Shang, Y., Nogal, M., Teixeira, R., & Wolfert, A. R. M. (2024). Extreme-oriented sensitivity analysis using sparse polynomial chaos expansion. Application to train–track–bridge systems. *Reliability Engineering & System Safety, 243*, 109818.

Furthermore, the methodology proposed in this work extends upon previous research (as shown below) by integrating limit state design considerations into the formulation of extreme-based engineering problems. Therefore, the code for this project is built upon the foundation of the following work.

* Nogal, M., & Nogal, A. (2021). Sensitivity method for extreme-based engineering problems. *Reliability Engineering & System Safety, 216,* 107997.

For case studies used in this project, a finite element model to simulate train-track-bridge dynamic interaction was employed, which is licensed under the [GNU General Public License v3.0](https://www.gnu.org/licenses/gpl-3.0.en.html). The details are referred to the following publication. 

* Cantero, D. (2022). TTB-2D: Train–Track–Bridge interaction simulation tool for Matlab. *SoftwareX, 20,* 101253.

## Prerequisites and installation

The system requirements to run the files are: MATLAB® R2020a or later, with toolboxes installed including Optimization toolbox and Global Optimization toolbox. 

In addition, the files are intended to work with UQLab version 2.0.0, the Framework for Uncertainty Quantification. The UQLab package should be properly installed before running the attached files. The basic installation is referred to this [website](https://www.uqlab.com/install). The readers are referred to the following publication for further information.

* Marelli, S., & Sudret, B. (2014). UQLab: A framework for uncertainty quantification in Matlab. In *Vulnerability, uncertainty, and risk: quantification, mitigation, and management* (pp. 2554-2563).

## Running the files
The zip file contains two sensitivity analyses for a train-track-bridge system that are based on the polynomial chaos expansion: one for extreme-based and the other for threshold-based sensitivity analysis. The data in .mat files are developed based on UQlab version 2.0.0, which are incorporated into the sensitivity calculations in matlab code files.

## Contact
Please contact [Yue Shang](y.shang@tudelft.nl) for further information and support of this repository.

## License

This project is licensed under the [GNU General Public License v3.0](https://www.gnu.org/licenses/gpl-3.0.en.html).