clc, clear
close all
% select "my_UQLab_folder/core" as your working directory; see README.md
rng(100,'twister')
uqlab
tic

%% Input - select one of the following 4 options
% output_lab='Sleeper acceleration (m/s^2)'; 

output_lab='Rail displacement (m)'; 

% output_lab='Sperling index'; 

% output_lab='Bridge displacement (m)'; 

var_labs={
    'M_c'        % x1: Main body mass [kg]
    'K_{pv}'    % x2: Primary suspension stiffness [N/m]                 
    'K_{sv}'    % x3: Secondary suspension stiffness [N/m] 
    'K_p'        % x4: Railpad vertical stiffness [N/m] 
    'M_s'        % x5: sleeper mass [kg]
    'rho_{ba}'    % x6: ballast density (kg/m^3)
    'E_{ba}'       % x7: ballast elastic modulus (N/m^2) 
    'H_{ba}'       % x8: ballast height (m) 
    'alpha'         % x9: ballast load distribution angle (degree)
    'C_{ba}'       % x10: ballast damping (Ns/m) 
    'C_p'        % x11: Railpad vertical damping [Ns/m] 
    'M_{br}'       % x12: Mass per unit length [kg/m]
    'E_{br}'       % x13: Modulus of elasticity [N/m^2]
    'C_{br}'       % x14: Damping [%]
    'I_{br}'       % x15: Second moment of area [m^4]
    'Irr'       % x16: irregularity amplitude
    };
 
% Definition of model variables: range
range=[
    22367.2471324521 44773.8153351932      
    946759.726879868 1505918.06324091
    339255.568798620 539620.639327993
    1e8 1.5e9
    220 325
    1500 2100
    8e7 1.6e8
    0.3 0.6
    15 35
    4e4 2.8e5  %
    1e4 7e4    %
    62579.2798676473 75420.7201323527
    28486225952.6856 41513774047.3144
    1.30209563778775 2.69790436221225
    45.3329177030852 57.2670822969148
    4.032e-7 10.8e-7];      
 
MinMax='Min';
done=0;  %calculate optimization
nsim_v=10*[20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20]; 

%% SAMPLING
nvars=length(nsim_v);
vars=cell(1,nvars);
seedtype=cell(1,nvars);
for v=1:nvars
    [vars{v},seedtype{v}]=Discretizing(var_labs{v},range(v,1),range(v,2),nsim_v(v),1e3);
end


%% CALCULATION OF EXTREME
if ~done
    t = cputime;
    
    %--- ABSOLUTE OPTIMAL: try the method that best approximates the solution and get the optimal solution x_is0
    rng(1711);
    x_is00=mean(range,2); 
       
    nvars=length(range);lb=range(:,1);ub=range(:,2);

    [x_is00,opt]=optimizationPS(MinMax,lb,ub,x_is00);
    optimals=[opt;x_is00]; 

    nsimul=sum(nsim_v);  
    Xs_vars=zeros(nsimul,nvars); %[var1, var2, var3, ...,output]
    
    pos=1;
    for v=1:nvars
        prevpoint=x_is00;
        for i=1:nsim_v(v) 
            Xs_vars(pos,v)=vars{v}(i); 
            
            lb=range(:,1);ub=range(:,2);
            [lb(v,:),ub(v,:)]=deal(vars{v}(i));  %value of x0 is fixed            
            
            x_is0=x_is00;
            [x_is0,Xs_vars(pos,nvars+1)]=optimizationPS(MinMax,lb,ub,x_is0);
            prevpoint=x_is0;
                                            
            fprintf('Evaluation %-2.0f out of %-5.0f (Optimal: %-2.8f)\n',pos,nsimul,Xs_vars(pos,nvars+1))
            pos=pos+1;
        end
    end
    comptime=cputime-t; fprintf('CPU time (seconds): %f\n',comptime)
    % Option 1 -> As
%     save('Optimal_sleeper_d20.mat','Xs_vars','optimals','comptime','vars')
    
    % Option 2 -> Ur
    save('Optimal_Ur_d20.mat','Xs_vars','optimals','comptime','vars')

    % Option 3 -> Wz
%     save('Optimal_Wz_d20.mat','Xs_vars','optimals','comptime','vars')

    % Option 4 -> Ud
%      save('Optimal_Ud_d20.mat','Xs_vars','optimals','comptime','vars')
else
    %load('Optimal_Ur_d20.mat','Xs_vars','optimals','comptime','vars')

end 

%% RESULTS
IM=ImportanceMeasure(nvars,Xs_vars,nsim_v);

fprintf('ABSOLUTE VALUE: %-8.4e\n',optimals(1))
for v=1:nvars,fprintf('     %s: %-8.3f\n',var_labs{v},optimals(v+1)),end

TableReport(var_labs,range,IM,ExtSens,vars)

toc

%%%%%%%%%%%%%%%%%%%%%%%%
%--- Objective Function
function Y = equation(x_is)
   
% % Option 1 -> As
%     persistent myModel_As_v3
%     if isempty(myModel_As_v3 )
%         load('myPCE_As_v3.mat','myModel_As_v3')
% 
%     end
%     if size(x_is,1)<size(x_is,2); x_is=x_is'; end
%     
%     % make predictions on a new sample for all limit state functions
%     As = uq_evalModel(myModel_As_v3,x_is'); 
%     Y=-As;

% Option 2 -> Ur
    persistent myModel_Ur_v3
    if isempty(myModel_Ur_v3 )
        load('myPCE_Ur_v3.mat','myModel_Ur_v3')

    end
    if size(x_is,1)<size(x_is,2); x_is=x_is'; end
    
    % make predictions on a new sample for all limit state functions
    Ur = uq_evalModel(myModel_Ur_v3,x_is'); 
    Y=-Ur;

% % Option 3 -> Wz
%     persistent myModel_Wz_v3
%     if isempty(myModel_Wz_v3 )
%         load('myPCE_Wz_v3.mat','myModel_Wz_v3')
% 
%     end
%     if size(x_is,1)<size(x_is,2); x_is=x_is'; end
%     
%     % make predictions on a new sample for all limit state functions
%     Wz = uq_evalModel(myModel_Wz_v3,x_is'); 
%     Y=-Wz;

% %Option 4 -> Ud
%     persistent myModel_Ud_v3
%     if isempty(myModel_Ud_v3 )
%         load('myPCE_Ud_v3.mat','myModel_Ud_v3')
% 
%     end
%     if size(x_is,1)<size(x_is,2); x_is=x_is'; end
%     
% %     make predictions on a new sample for all limit state functions
%     Ud = uq_evalModel(myModel_Ud_v3,x_is'); 
%     Y=-Ud;
end

function [x_is,z]=optimizationPS(MinMax,lb,ub,x0)
A_lin=[]; b_lin=[]; %linear equalities
Aeq=[]; beq=[]; %linear inequalities

nevals=4000*length(lb); 
options = optimoptions('patternsearch','MaxFunctionEvaluations',nevals,'UseCompletePoll',true);
[x_is,z]=patternsearch(@equation,x0,A_lin,b_lin,Aeq,beq,lb,ub,'',options);
if strcmp(MinMax,'Max'), z=-z; end
 
end

function TableReport(var_labs,range,IM,ExtSens,vars)
%--- range of the variables
[nvars,~]=size(range);
Rang=cell(nvars,1);
for i=1:nvars
    Rang{i}=sprintf('(%.4f, %.4f)',range(i,1),range(i,2));
end
%--- sensitivity by derivatives
sen_value_ratio=zeros(nvars,3);
for v=1:nvars
    [~, pos]=max(abs(ExtSens{v})); %identify the maximum slope (negative or positive)
    sen_value_ratio(v,1)=ExtSens{v}(pos); %sensitivity
    sen_value_ratio(v,2)=vars{v}(pos); %value of the var
    sen_value_ratio(v,3)=(vars{v}(pos)-vars{v}(1))/(vars{v}(end)-vars{v}(1)); % relative range    
end

%--- Definition of the table for maximum values
% sort from max to min (for maximum)
[~,ind]=sort(IM,'descend');
%
Variables00=var_labs(ind);
Range=Rang(ind);
Local_Sens=sen_value_ratio(ind,1);
Value_Var=sen_value_ratio(ind,2);
Relat_pos=sen_value_ratio(ind,3);
Indices=IM(ind)';
table(Variables00,Range,Indices,Local_Sens,Value_Var,Relat_pos)

end
 
 
function [dist,typeS]=Discretizing(varname,range1, range2, dist_length,threshold)
if (range2-range1)>threshold
    dist=logspace(log10(range1),log10(range2),dist_length);
    typeS='log';
    fprintf('%s: Log\n',varname)
    
else  
    dist=linspace(range1,range2,dist_length);
    typeS='lin';
    fprintf('%s: Lin\n',varname)
end
end
 
function IM=ImportanceMeasure(nvars,Xs_vars,nsim_v)
 
varY_v=zeros(1,nvars);
Weights=zeros(sum(nsim_v),1);
pos=0;
for v=1:nvars  
   
    resp=Xs_vars(pos+1:pos+nsim_v(v),nvars+1); %(Max/Min of Y for ~Xi para cada Xi)
    Ws=Weights_CorrDisc(Xs_vars(pos+1:pos+nsim_v(v),v))';
   
    %save vales
    Weights(pos+1:pos+nsim_v(v))=Ws;
    pos=pos+nsim_v(v);
 
    %mean and variance of each variable
    W_Mean=sum(resp.*Ws/sum(Ws));
    varY_v(v)=sum( Ws/sum(Ws).*(resp-W_Mean).^2);
end
IM=varY_v/sum(varY_v);  %Importance measure
 
end
 
function Ws=Weights_CorrDisc(disct)
%disct is the x positions as a result of the discretization
disct=disct';
 
% weight is claculated using distance between points as a comparison with the uniform distribution (if s<1/n weights less, if s>1/n weight more)
s=[0.5*(disct(2)-disct(1)) , 0.5*(disct(3:end)-disct(1:end-2)), 0.5*(disct(end)-disct(end-1))];
Ws=s;
 
end
 

